/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2016 T-Plan Limited (http://www.t-plan.com),
 * All rights reserved.
 */
package com.tplan.extent;

import com.tplan.robot.plugin.PluginManager;
import com.tplan.robot.util.Utils;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;

/**
 * Factory producing Extent report generators. It takes care of dynamic loading
 * of the Extent libraries if they are not present on the class path.
 */
class ExtentReportFactory {

    private static ExtentReportFactory instance;
    private static final String[] IMPL_CLASSES = new String[]{
        ExtentReportFactory.class.getPackage().getName() + ".KlovReportImpl",
        ExtentReportFactory.class.getPackage().getName() + ".ExtentReportImpl"
    };

    private ExtentReportFactory() {
    }

    public static synchronized ExtentReportFactory getInstance() {
        if (instance == null) {
            instance = new ExtentReportFactory();
        }
        return instance;
    }

    public ExtentReport getReportInstance() throws UnsupportedOperationException {
        Class cl = null;
        try {
            for (String clazz : IMPL_CLASSES) {
                try {
                    cl = Class.forName(clazz);
                    return (ExtentReport) cl.newInstance();
                } catch (Throwable ct) {
                }
            }
            throw new IllegalStateException();
        } catch (Throwable t) {
            // Class not found or similar -> load all JARs from the current 
            // folder and the lib/ subfolder
            Throwable err = null;
            try {
                File jar = Utils.getJarFile(getClass());
                if (jar != null) {
                    File dir = jar.getParentFile();
                    loadJarsFromFolder(dir);

                    dir = new File(dir, "lib");
                    loadJarsFromFolder(dir);
                    for (String clazz : IMPL_CLASSES) {
                        try {
                            cl = Class.forName(clazz);
                            return (ExtentReport) cl.newInstance();
                        } catch (Throwable ct) {
                            if (err == null) {
                                err = ct;
                            }
                        }
                    }
                }
                throw err;
            } catch (Throwable t2) {
                throw new UnsupportedOperationException(t2.getMessage(), t2);
            }
        }
    }

    /**
     * Load all JAR files from the specified folder.
     *
     * @param dir a folder.
     * @throws MalformedURLException
     * @throws IOException
     */
    private void loadJarsFromFolder(File dir) throws MalformedURLException, IOException {
        File jars[] = dir.listFiles();
        if (jars != null && jars.length > 0) {
            for (File f : jars) {
                if (f.isFile() && f.getName().toLowerCase().endsWith(".jar")) {
                    PluginManager.addURL(f.toURI().toURL());
                }
            }
        }
    }
}
